@@ -1,8 +1,5 @@
 #![warn(clippy::all)]
 #![allow(clippy::too_many_arguments)]
-#![allow(clippy::many_single_char_names)]
-#![allow(clippy::suspicious_operation_groupings)]
-
 mod dft;
 mod eos;
 mod parameters;
