@@ -7,7 +7,7 @@ use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::fmt::Write;
 
-/// PcSaft parameter set.
+/// PC-SAFT pure-component parameters.
 #[derive(Serialize, Deserialize, Clone, Default)]
 pub struct PcSaftRecord {
     /// Segment number
@@ -17,29 +17,30 @@ pub struct PcSaftRecord {
     /// Energetic parameter in units of Kelvin
     pub epsilon_k: f64,
     /// Dipole moment in units of Debye
-    #[serde(default)]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub mu: Option<f64>,
     /// Quadrupole moment in units of Debye
-    #[serde(default)]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub q: Option<f64>,
-    #[serde(default)]
+    /// Association volume parameter
     #[serde(skip_serializing_if = "Option::is_none")]
     pub kappa_ab: Option<f64>,
-    #[serde(default)]
+    /// Association energy parameter in units of Kelvin
     #[serde(skip_serializing_if = "Option::is_none")]
     pub epsilon_k_ab: Option<f64>,
-    #[serde(default)]
+    /// \# of association sites of type A
     #[serde(skip_serializing_if = "Option::is_none")]
     pub na: Option<f64>,
-    #[serde(default)]
+    /// \# of association sites of type B
     #[serde(skip_serializing_if = "Option::is_none")]
     pub nb: Option<f64>,
+    /// Entropy scaling coefficients for the viscosity
     #[serde(skip_serializing_if = "Option::is_none")]
     pub viscosity: Option<[f64; 4]>,
+    /// Entropy scaling coefficients for the diffusion coefficient
     #[serde(skip_serializing_if = "Option::is_none")]
     pub diffusion: Option<[f64; 5]>,
+    /// Entropy scaling coefficients for the thermal conductivity
     #[serde(skip_serializing_if = "Option::is_none")]
     pub thermal_conductivity: Option<[f64; 4]>,
 }
@@ -56,33 +57,85 @@ impl FromSegments for PcSaftRecord {
             epsilon_k += s.m * s.epsilon_k * *n;
         });
 
-        // We do not allow more than a single segment for q, mu, kappa_ab, epsilon_k_ab
-        let q: Vec<f64> = segments.iter().filter_map(|s| s.0.q).collect();
-        let q = match q.len() {
-            0 => None,
-            1 => Some(q[0]),
-            _ => panic!("More than one segment with quadrupole moment."),
+        let q = segments
+            .iter()
+            .filter_map(|(s, n)| s.q.map(|q| q * *n))
+            .reduce(|a, b| a + b);
+        let mu = segments
+            .iter()
+            .filter_map(|(s, n)| s.mu.map(|mu| mu * *n))
+            .reduce(|a, b| a + b);
+        let kappa_ab = segments
+            .iter()
+            .filter_map(|(s, n)| s.kappa_ab.map(|k| k * *n))
+            .reduce(|a, b| a + b);
+        let epsilon_k_ab = segments
+            .iter()
+            .filter_map(|(s, n)| s.epsilon_k_ab.map(|e| e * *n))
+            .reduce(|a, b| a + b);
+        let na = segments
+            .iter()
+            .filter_map(|(s, n)| s.na.map(|na| na * *n))
+            .reduce(|a, b| a + b);
+        let nb = segments
+            .iter()
+            .filter_map(|(s, n)| s.nb.map(|nb| nb * *n))
+            .reduce(|a, b| a + b);
+
+        // entropy scaling
+        let mut viscosity = if segments
+            .iter()
+            .all(|(record, _)| record.viscosity.is_some())
+        {
+            Some([0.0; 4])
+        } else {
+            None
         };
-        let mu: Vec<f64> = segments.iter().filter_map(|s| s.0.mu).collect();
-        let mu = match mu.len() {
-            0 => None,
-            1 => Some(mu[0]),
-            _ => panic!("More than one segment with dipole moment."),
+        let mut thermal_conductivity = if segments
+            .iter()
+            .all(|(record, _)| record.thermal_conductivity.is_some())
+        {
+            Some([0.0; 4])
+        } else {
+            None
         };
-        let kappa_ab: Vec<f64> = segments.iter().filter_map(|s| s.0.kappa_ab).collect();
-        let kappa_ab = match kappa_ab.len() {
-            0 => None,
-            1 => Some(kappa_ab[0]),
-            _ => panic!("More than one segment with association site."),
+        let diffusion = if segments
+            .iter()
+            .all(|(record, _)| record.diffusion.is_some())
+        {
+            Some([0.0; 5])
+        } else {
+            None
         };
-        let epsilon_k_ab: Vec<f64> = segments.iter().filter_map(|s| s.0.epsilon_k_ab).collect();
-        let epsilon_k_ab = match epsilon_k_ab.len() {
-            0 => None,
-            1 => Some(epsilon_k_ab[0]),
-            _ => panic!("More than one segment with association site"),
-        };
-        let na = Some(1.0);
-        let nb = Some(1.0);
+
+        let n_t = segments.iter().fold(0.0, |acc, (_, n)| acc + n);
+        segments.iter().for_each(|(s, n)| {
+            let s3 = s.m * s.sigma.powi(3) * *n;
+            if let Some(p) = viscosity.as_mut() {
+                let [a, b, c, d] = s.viscosity.unwrap();
+                p[0] += s3 * a;
+                p[1] += s3 * b / sigma3.powf(0.45);
+                p[2] += *n * c;
+                p[3] += *n * d;
+            }
+            if let Some(p) = thermal_conductivity.as_mut() {
+                let [a, b, c, d] = s.thermal_conductivity.unwrap();
+                p[0] += *n * a;
+                p[1] += *n * b;
+                p[2] += *n * c;
+                p[3] += n_t * d;
+            }
+            // if let Some(p) = diffusion.as_mut() {
+            //     let [a, b, c, d, e] = s.diffusion.unwrap();
+            //     p[0] += s3 * a;
+            //     p[1] += s3 * b / sigma3.powf(0.45);
+            //     p[2] += *n * c;
+            //     p[3] += *n * d;
+            // }
+        });
+        // correction due to difference in Chapman-Enskog reference between GC and regular formulation.
+        viscosity = viscosity.map(|v| [v[0] - 0.5 * m.ln(), v[1], v[2], v[3]]);
+
         Self {
             m,
             sigma: (sigma3 / m).cbrt(),
@@ -93,9 +146,9 @@ impl FromSegments for PcSaftRecord {
             epsilon_k_ab,
             na,
             nb,
-            viscosity: None,
-            diffusion: None,
-            thermal_conductivity: None,
+            viscosity,
+            diffusion,
+            thermal_conductivity,
         }
     }
 }
@@ -186,6 +239,7 @@ impl FromSegmentsBinary for PcSaftBinaryRecord {
     }
 }
 
+/// Parameter set required for the PC-SAFT equation of state and Helmholtz energy functional.
 pub struct PcSaftParameters {
     pub molarweight: Array1<f64>,
     pub m: Array1<f64>,
