@@ -23,7 +23,7 @@ fn test_stability_analysis() -> Result<(), Box<dyn Error>> {
         DensityInitialization::Liquid,
     )?;
     let check = unstable.stability_analysis(Default::default())?;
-    assert!(check.len() > 0);
+    assert!(!check.is_empty());
 
     let params = PcSaftParameters::from_json(
         vec!["propane", "butane"],
@@ -32,17 +32,17 @@ fn test_stability_analysis() -> Result<(), Box<dyn Error>> {
         IdentifierOption::Name,
     )?;
     let mix = Rc::new(PcSaft::new(Rc::new(params)));
-    let vle = PhaseEquilibrium::bubble_point_tx(
+    let vle = PhaseEquilibrium::bubble_point(
         &mix,
         300.0 * KELVIN,
-        Some(6.0 * BAR),
         &arr1(&[0.5, 0.5]),
+        Some(6.0 * BAR),
         None,
         Default::default(),
     )?;
     let vapor_check = vle.vapor().stability_analysis(Default::default())?;
     let liquid_check = vle.liquid().stability_analysis(Default::default())?;
-    assert!(vapor_check.len() == 0);
-    assert!(liquid_check.len() == 0);
+    assert!(vapor_check.is_empty());
+    assert!(liquid_check.is_empty());
     Ok(())
 }
